package com.yifu.cloud.plus.v1.yifu.salary.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.yifu.cloud.plus.v1.yifu.admin.api.entity.SysUser;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.CommonConstants;
import com.yifu.cloud.plus.v1.yifu.common.core.exception.CheckedException;
import com.yifu.cloud.plus.v1.yifu.common.core.util.Common;
import com.yifu.cloud.plus.v1.yifu.common.core.util.R;
import com.yifu.cloud.plus.v1.yifu.common.core.vo.YifuUser;
import com.yifu.cloud.plus.v1.yifu.salary.entity.TAuditRecord;
import com.yifu.cloud.plus.v1.yifu.salary.entity.TAuditNodeProocess;
import com.yifu.cloud.plus.v1.yifu.salary.service.TAuditRecordService;
import com.yifu.cloud.plus.v1.yifu.salary.service.TAuditNodeProocessService;
import com.yifu.cloud.plus.v1.yifu.process.constant.ProcessConstants;
import com.yifu.cloud.plus.v1.yifu.process.entity.*;
import com.yifu.cloud.plus.v1.yifu.process.util.CommonProcessUserUtil;
import com.yifu.cloud.plus.v1.yifu.process.util.ProcessAuditUtil;
import com.yifu.cloud.plus.v1.yifu.process.vo.ProcessVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;

/**
 * 流程审批抽象类——仿照crm流程处理
 * @author 陈宇茜
 * @date 2025/01/13
 * @since 1.7.5
 */
@Slf4j
public abstract class AbastractAuditProcess<T,E extends T> {

	@Resource
	protected TAuditNodeProocessService auditNodeProocessService;

	@Resource
	protected TAuditRecordService approvalRecordService;

    public static final String BIGIN_NODE_NO_FOUND = "未找到开始审批节点，请联系管理员";
    public static final String NEXT_NODE_USER_NULL = "未查询到下个节点的用户，请联系管理员";
    public static final String NEXT_AUDIT_NODE_NOT_FOUND = "未查询到下一个审核节点，请联系管理员";

    /**
     * 审核成功并结束逻辑
	 * @param entity 业务实体
	 * @param approvalRecord 审批记录
	 * @param loginUser 当前登录用户
     * @return R<String>
     **/
    abstract public R<String> auditSuccess(T entity, TAuditRecord approvalRecord, YifuUser loginUser);

    /**
	 * 审核成功并结束逻辑
	 * @param entity 业务实体
	 * @param approvalRecord 审批记录
	 * @param remoteSysUserList 当前登录用户
	 * @return R<String>
     **/
    abstract public R<String> auditSuccessAndNex(T entity, List<SysUser> remoteSysUserList, TAuditRecord approvalRecord);

    /**
	 * 审核不通过处理逻辑
	 * @param entity 业务实体
	 * @param approvalRecord 审批记录
	 * @return R<String>
     **/
    abstract public R<String> auditFail(T entity, TAuditRecord approvalRecord);

    /**
     * 选择节点。不同类型可能有不同选择节点
	 * @param entity 业务实体
     * @return TProcessNode
     **/
    abstract public TProcessNode choiceGetNode(ProcessVo processVo, List<TNodeRoute> nodeRouteList, List<TNodeRouteCondition> nodeRouteConditionList,
											   TProcessNode currentNode, T entity, String type, YifuUser loginUser);

    /**
     * 审批发消息
	 * @param users  发送人
     * @param entity 实体信息
     * @param type 1发送审批消息 2发送抄送消息
     * @return R<String>
     **/
    abstract public R<String> sendMessage(List<SysUser> users, T entity, String type, TAuditRecord approvalRecord);

	/**
	 * 根据事业部条线获取审批用户
	 * @param nodePrimission
	 * @param entity
	 * @return List<SysUser>
	 **/
    abstract public List<SysUser> getDivisionAuditUser(TNodePrimission nodePrimission, T entity);


    /**
     * 获取当前节点的审批人
	 * @param sysUserTprocessNode 当前节点
     * @param departId              部门id
     * @param entityId            实体id
     * @return List<SysUser>
     **/
    private List<SysUser> getAuditUserByTprocessNode(TProcessNode sysUserTprocessNode, String departId, String entityId, String auditType,
													 T entity, TAuditRecord approvalRecord) {
        // todo 获取当前流程 改成dapr写法
        R<ProcessVo> processVoByName = null;
				// remoteProcessService.getProcessVoByName(sysUserTprocessNode.getProcessId());
        // todo 临时
		if(processVoByName == null){
			throw new CheckedException("未找到审批节点,请联系管理员");
		}


        ProcessVo processVo = processVoByName.getData();
        List<TNodePrimission> nodePrimissions = null;
        for (TProcessNode processNode : processVo.getProcessNodeList()) {
            if (StringUtils.equals(sysUserTprocessNode.getLogoTitle(), processNode.getLogoTitle())) {
                List<TNodePrimission> nodePrimissionList = processNode.getNodePrimissionList();
                if (nodePrimissionList != null && !nodePrimissionList.isEmpty()) {
                    nodePrimissions = nodePrimissionList;
                }
            }
        }

        if (nodePrimissions == null) {
            throw new CheckedException("未找到审批节点,请联系管理员");
        }
        // 获取所有审批用户
        List<SysUser> sysUserList = new ArrayList<>();
		// 节点是否允许跳过（0：不允许，1允许）
		Integer isPass = CommonConstants.ZERO_INT;
        for (TNodePrimission nodePrimission : nodePrimissions) {
			// 审核用户改造； 2指定权限;3指定用户;5指定角色(crm)；6指定事业部配置表
			if (CommonConstants.THREE_INT == nodePrimission.getType().intValue()
					|| CommonConstants.TWO_INT == nodePrimission.getType().intValue()
					|| CommonConstants.FIVE_INT == nodePrimission.getType().intValue()) {
				// todo
				sysUserList = CommonProcessUserUtil.getDepartAuditUser(nodePrimission, departId);
			}
			if (CommonConstants.SIX_INT == nodePrimission.getType().intValue()) {
				// todo 根据事业部条线获取审批用户
				sysUserList = this.getDivisionAuditUser(nodePrimission, entity);
			}

            if (Common.isEmpty(sysUserList)) {
                throw new CheckedException("未找到审批用户,请联系管理员");
            }

			// 事业部条线没有审批人直接审核通过,配置了可以跳过
			if(Common.isEmpty(sysUserList) && CommonConstants.ONE_STRING.equals(nodePrimission.getIsPass())){
				// 标记节点可以跳过
				isPass = CommonConstants.ONE_INT;
				// todo 系统自动给当前节点完成审核，将审核通过
				// sysAutoAudit(sysUserTprocessNode,entity,processVo);
			} else {
				if (Common.isEmpty(sysUserList)) {
					throw new CheckedException("未找到审批用户");
				}
				//微信审批通知发送
				//发送审批消息
				sendMessage(sysUserList,entity,CommonConstants.ONE_STRING,approvalRecord);
				for (SysUser sysUser : sysUserList) {
					// 新增指派审核流程记录
					TAuditRecord newEntity = new TAuditRecord();
					newEntity.setDomainId(entityId);
					newEntity.setFlowId(sysUserTprocessNode.getProcessId());
					newEntity.setNodeId(sysUserTprocessNode.getId());
					newEntity.setApprovalMan(String.valueOf(sysUser.getUserId()));
					newEntity.setApprovalManName(sysUser.getNickname());
					newEntity.setDeleteFlag(CommonConstants.NOT_DELETE_FLAG);
					newEntity.setPreFlag(CommonConstants.ONE_STRING);
					newEntity.setApprovalResult(CommonConstants.TWO_STRING);
					newEntity.setNodeName(sysUserTprocessNode.getName());
					// 获取提交人信息
					TAuditRecord tAuditRecord = approvalRecordService.getOne(Wrappers.<TAuditRecord>query().lambda().eq(TAuditRecord::getDomainId, entityId)
							.eq(TAuditRecord::getAuditType, auditType)
							.eq(TAuditRecord::getPreFlag, CommonConstants.ONE_STRING)
							.eq(TAuditRecord::getApprovalResult, CommonConstants.THREE_STRING)
							.orderByDesc(TAuditRecord::getId).last(CommonConstants.SQL_LIMIT_ONE));
					if(Common.isNotNull(tAuditRecord)) {
						newEntity.setSubmitMan(tAuditRecord.getSubmitMan());
						newEntity.setSubmitManName(tAuditRecord.getSubmitManName());
						newEntity.setSubmitCreateTime(tAuditRecord.getSubmitCreateTime());
						newEntity.setReason(tAuditRecord.getReason());
						newEntity.setEntityName(tAuditRecord.getEntityName());
						newEntity.setAuditType(tAuditRecord.getAuditType());
					}
					newEntity.setCreateTime(LocalDateTime.now());
					approvalRecordService.save(newEntity);
				}
				// todo 存当前审核人信息[AUDIT_USER]多个用逗号隔开

			}

        }
		// 事业部条线没有审批人直接审核通过,配置了可以跳过
		if(Common.isEmpty(sysUserList) && isPass == CommonConstants.ONE_INT){
			sysUserList = new ArrayList<>();
			SysUser sysUserTemp = new SysUser();
			sysUserList.add(sysUserTemp);
		}

        return sysUserList;
    }


    /**
     * 提交审核：获取审核人
	 * @param id          实体id
     * @param loginUser   当前登录人
     * @param departId    部门id      通过该部门获取审批流程
     * @param processType 流程类型 1不购买商险申请
     * @param auditType   审核类型 0不购买商险审核
     * @return R<List<SysUser>>
     **/
    public R<List<SysUser>> commitAuditUserList(String id, YifuUser loginUser, String departId, String processType, String auditType,
												String reason, String entityName, T entity, Class<T> clazz) {

        //获取合同审批流程与下一个审核人节点
        ProcessVo processVo = null;
        //获取正确流程
        TProcessApprovalConf processApprovalConf = getProcessApprovalConf(processType);
        // todo 改成dapr获取
        //processVo = ProcessAuditUtil.getProcessVo(remoteProcessService, processApprovalConf.getProcessId());
        if (processVo == null) {
            throw new CheckedException("未找到满足条件的主流程,请联系管理员");
        }
        //获取开始节点
        TProcessNode startTProcessNode = ProcessAuditUtil.getBeginProcessNode(processVo);
        if (startTProcessNode == null) {
            return R.failed(BIGIN_NODE_NO_FOUND);
        }
        String currentNode = startTProcessNode.getLogoTitle();
        // 保存TAuditNodeProocess节点信息
        saveNodeProcess(processVo, currentNode, id, auditType);
        // 获取开始节点下一个审核节点
        TProcessNode sysUserTprocessNode = getNextNode(processVo, currentNode, id, auditType, entity, loginUser);
        if (sysUserTprocessNode == null) {
            throw new CheckedException(NEXT_AUDIT_NODE_NOT_FOUND);
        }

        // 记录提交审核日志
        TAuditRecord submitAuditRecord = new TAuditRecord();
        submitAuditRecord.setDomainId(id);
        submitAuditRecord.setFlowId(startTProcessNode.getProcessId());
        submitAuditRecord.setNodeId(startTProcessNode.getId());
        submitAuditRecord.setApprovalMan(String.valueOf(loginUser.getId()));
        submitAuditRecord.setApprovalManName(loginUser.getNickname());
        // 审批结果为“提交”
        submitAuditRecord.setApprovalResult(CommonConstants.dingleDigitStrArray[3]);
        submitAuditRecord.setDeleteFlag(CommonConstants.NOT_DELETE_FLAG);
        submitAuditRecord.setApprovalTime(LocalDateTime.now());
		// 审核记录标识 1当前审核记录
        submitAuditRecord.setPreFlag(CommonConstants.ONE_STRING);
        submitAuditRecord.setSubmitMan(String.valueOf(loginUser.getId()));
        submitAuditRecord.setSubmitManName(loginUser.getNickname());
        //申请原因
        submitAuditRecord.setReason(reason);
        //审核类型
        submitAuditRecord.setAuditType(auditType);
        submitAuditRecord.setCreateTime(LocalDateTime.now());
        submitAuditRecord.setNodeName("提交");
        submitAuditRecord.setSubmitCreateTime(LocalDateTime.now());
        submitAuditRecord.setEntityName(entityName);
        approvalRecordService.save(submitAuditRecord);

        // 获取节点审核人
        List<SysUser> remoteSysUserList = getAuditUserByTprocessNode(sysUserTprocessNode, departId, id, auditType,entity, submitAuditRecord);
        if (remoteSysUserList == null) {
            throw new CheckedException(NEXT_NODE_USER_NULL);
        }
        return new R<>(remoteSysUserList);

    }

    /**
     * 审核并提交下一个节点：获取下一个审核人
	 * @param id             实体id
     * @param approvalResult    审核标识 0审核通过 其他审核不通过
     * @param approvalOpinion 审核意见
     * @param loginUser      当前登录人
     * @param auditType      审核类型 0不购买商险审核
     * @param entity         实体类：用于审核通过逻辑
     * @return R<String>
     **/
    public R<String> auditUserList(String id, String approvalResult, String approvalOpinion, YifuUser loginUser,
								   String auditType, T entity, String departId) {
        String loginUserId = loginUser.getId();
        // 判断当前审核是否为最新待审核记录
        List<TAuditRecord> approvalRecords = approvalRecordService.list
                (Wrappers.<TAuditRecord>query().lambda().eq(TAuditRecord::getDomainId, id)
                        .eq(TAuditRecord::getAuditType, auditType)
                        .eq(TAuditRecord::getDeleteFlag, CommonConstants.NOT_DELETE_FLAG)
                        .eq(TAuditRecord::getApprovalResult, CommonConstants.TWO_STRING)
                        .eq(TAuditRecord::getApprovalMan, loginUserId));
        if (approvalRecords.size() != CommonConstants.ONE_INT) {
            return R.failed("当前用户不能审核该记录");
        }

        // 更新审核记录信息
        TAuditRecord approvalRecord = approvalRecords.get(0);
        approvalRecord.setApprovalResult(approvalResult);
        approvalRecord.setApprovalOpinion(approvalOpinion);
        approvalRecord.setApprovalTime(LocalDateTime.now());
        approvalRecordService.updateById(approvalRecord);
        // 移除同一个节点的另外审批人。多个节点一个人审批就够了,其他都移除
        approvalRecordService.remove(Wrappers.<TAuditRecord>query().lambda()
                .eq(TAuditRecord::getDomainId, id)
                .eq(TAuditRecord::getAuditType, auditType)
                .isNull(TAuditRecord::getApprovalTime));

        // 审核通过处理
        if (CommonConstants.ZERO_STRING.equals(approvalResult)) {
            // 获取流程，并判断是否是最后一个节点
            // 获取流程的主流程
            TAuditNodeProocess auditNodeProocess = auditNodeProocessService.getOne(Wrappers.<TAuditNodeProocess>query().lambda()
                    .eq(TAuditNodeProocess::getDomainId, id)
                    .eq(TAuditNodeProocess::getType, auditType)
                    .eq(TAuditNodeProocess::getMainType, CommonConstants.ZERO_STRING)
					.eq(TAuditNodeProocess::getDeleteFlag, CommonConstants.NOT_DELETE_FLAG)
                    .orderByDesc(TAuditNodeProocess::getId)
                    .last(CommonConstants.SQL_LIMIT_ONE));
            if (auditNodeProocess == null) {
                throw new CheckedException("未获取到审批记录流程数据");
            }
			// todo 改成dapr获取
            ProcessVo processVo = null;
					// ProcessAuditUtil.getProcessVo(remoteProcessService, auditNodeProocess.getProcessId());
			// todo 临时
			if (processVo == null) {
				throw new CheckedException("未获取到结束节点");
			}
            TProcessNode endProcessNode = ProcessAuditUtil.getEndProcessNode(processVo);
			if (endProcessNode == null) {
				throw new CheckedException("未获取到结束节点");
			}

			TProcessNode preEndTProcessNode = ProcessAuditUtil.getPreEndTProcessNode(processVo, endProcessNode);
			if (preEndTProcessNode == null) {
				throw new CheckedException("未找到结束节点前一个节点");
			}
			// 当前是结束节点前的一个节点
			if (StringUtils.equals(preEndTProcessNode.getId(), approvalRecord.getNodeId())) {
				// 走审核通过处理
				return auditSuccess(entity, approvalRecord, loginUser);
			} else {
				// todo 改成dapr——获取自动下一个审核人
				// 获取流程的下一个节点,可能下一个节点是子流程,也可能是主节点，所以重新获取当前执行审批的流程和节点
				ProcessVo processVoByApproval = null;
				// ProcessAuditUtil.getProcessVo(remoteProcessService, approvalRecord.getFlowId());
				// todo 临时
				if (processVoByApproval == null) {
					throw new CheckedException("未获取到下一个审核人节点");
				}

				// todo 判断当前是否是当前审核节点到下一个审核节点前的加签节点
				TProcessNode currentProcessNode = ProcessAuditUtil.getCurrentProcessNode(processVoByApproval, approvalRecord.getNodeId());
                TProcessNode sysUserTprocessNode = getNextNode(processVoByApproval, currentProcessNode.getLogoTitle(), id, auditType, entity, loginUser);
				if (sysUserTprocessNode == null) {
					throw new CheckedException(NEXT_AUDIT_NODE_NOT_FOUND);
				}

                auditNodeProocess = auditNodeProocessService.getOne(Wrappers.<TAuditNodeProocess>query().lambda()
                                .eq(TAuditNodeProocess::getDomainId, id)
                                .eq(TAuditNodeProocess::getType, auditType)
                                .eq(TAuditNodeProocess::getMainType, CommonConstants.ZERO_STRING)
                                .eq(TAuditNodeProocess::getDeleteFlag, CommonConstants.NOT_DELETE_FLAG)
                                .orderByDesc(TAuditNodeProocess::getId)
                                .last(CommonConstants.SQL_LIMIT_ONE));
				// 如果返回的是end节点：判断是不是返回的是不是主流程结束节点
				if (StringUtils.equals(ProcessConstants.PROCESS_NODE_END, sysUserTprocessNode.getType())
						&& StringUtils.equals(auditNodeProocess.getProcessId(), sysUserTprocessNode.getProcessId())) {
					//审核通过流程
					return auditSuccess(entity, approvalRecord, loginUser);
				}
				// 获取审批用户
				List<SysUser> remoteSysUserList = getAuditUserByTprocessNode(sysUserTprocessNode, departId, id, auditType, entity, approvalRecord);
				if (remoteSysUserList == null) {
					throw new CheckedException(NEXT_NODE_USER_NULL);
				}
				// 流程中的审核结束，继续下一个节点
				return auditSuccessAndNex(entity, remoteSysUserList, approvalRecord);
            }
        } else {
            //审核不通过处理
            return auditFail(entity, approvalRecord);
        }
    }

    /**
     * 获取下一个节点
	 * @param processVo         流程记录
     * @param currentNodeString 当前节点
     * @param id                实体id
     **/
    private TProcessNode getNextNode(ProcessVo processVo, String currentNodeString, String id, String auditType, T entity, YifuUser loginUser) {

        // 判断进行到的节点当前流程是主流程还是子流程,如果是主流程,则进行主流程，如果是子流程，则进行子流程判断
        TAuditNodeProocess auditNodeProocess = auditNodeProocessService.getOne(Wrappers.<TAuditNodeProocess>query().lambda()
                .eq(TAuditNodeProocess::getDomainId, id)
                .eq(TAuditNodeProocess::getType, auditType)
                .eq(TAuditNodeProocess::getDeleteFlag, CommonConstants.ZERO_STRING)
                .isNotNull(TAuditNodeProocess::getLogoTitle)
                .orderByDesc(TAuditNodeProocess::getId).last(CommonConstants.SQL_LIMIT_ONE));
        // 流程预览功能无需判断当前节点是否一致
        if (auditNodeProocess != null && !StringUtils.equals(currentNodeString, auditNodeProocess.getLogoTitle())) {
            throw new CheckedException("记录的流程节点和当前执行节点不匹配");
        }
        List<TProcessNode> processNodeList = processVo.getProcessNodeList();
        List<TNodeRoute> nodeRouteList = processVo.getNodeRouteList();
        //判断当前节点类型,如果是选择节点则进行条件表达式判断判断
        TProcessNode currentNode = null;
        for (TProcessNode processNode : processNodeList) {
            if (StringUtils.equals(currentNodeString, processNode.getLogoTitle())) {
                currentNode = processNode;
            }
        }
        if (currentNode == null) {
            throw new CheckedException("未获取到当前节点");
        }

        if (StringUtils.equals(ProcessConstants.PROCESS_NODE_TYPE_CHOICE, currentNode.getType())) {
            // 当前节点为选择节点,获取当前节点路由类型
            List<TNodeRouteCondition> nodeRouteConditionList = Lists.newArrayList();
            for (TNodeRoute nodeRoute : nodeRouteList) {
                if (StringUtils.equals(currentNode.getLogoTitle(), nodeRoute.getPrewNode())) {
                    nodeRouteConditionList.addAll(nodeRoute.getNodeRouteConditionList());
                }
            }
            return choiceGetNode(processVo, nodeRouteList, nodeRouteConditionList, currentNode, entity, auditType, loginUser);
        } else if (StringUtils.equals(ProcessConstants.PROCESS_NODE_END, currentNode.getType())) {
			TProcessNode endNode = ProcessAuditUtil.getEndProcessNode(processVo);
			if (StringUtils.equals(currentNode.getId(), endNode.getId())) {
				return endNode;
			} else {
				throw new CheckedException("当前不是结束节点");
			}
        } else if (StringUtils.equals(ProcessConstants.PROCESS_NODE_BEGIN, currentNode.getType()) || StringUtils.equals(ProcessConstants.PROCESS_NODE_SERVICE, currentNode.getType())) {
            //查找下个节点
            try {
                TNodeRoute nexTNodeRoute = ProcessAuditUtil.getNexTNodeRoute(nodeRouteList, currentNodeString);
                saveNodeProcess(processVo, nexTNodeRoute.getNextNode(), id, auditType);
                return validateTwoNextNode(processVo, nexTNodeRoute.getNextNode(), id, nexTNodeRoute.getRouteType(), auditType, entity, loginUser);
            }catch (Exception e){
                if (Common.isEmpty(processVo.getIsPreview())){
                    throw e;
                }
                return null;
            }
        } else {
            throw new CheckedException("未知的流程节点类型");
        }
    }

    /**
     * 通过表中配置的字段名称去取对应判断条件
	 * @param type     流程类型：类型  1不购买商险申请
     * @return TProcessApprovalConf
     **/
    private TProcessApprovalConf getProcessApprovalConf(String type) {
    	return null;
        // todo 找到对应的流程查看判断的类名和字段名
//        R<TProcessApprovalConf> processApprovalConfR = remoteProcessService.getRecordbyTypeAndParentId(type);
//        if (processApprovalConfR != null && processApprovalConfR.getData() != null && CommonConstants.SUCCESS == processApprovalConfR.getCode()) {
//            return processApprovalConfR.getData();
//        } else {
//            throw new CheckedException("未获取到对应流程配置");
//        }
//        throw new CheckedException("调用流程服务返回数据为空");
    }

    /**
     * 校验下一个节点
	 * @param processVo         流程配置信息
     * @param currentNodeString 当前节点
     * @param id                实体id
     * @param routeType         上一个路由类型
     * @return TProcessNode
     **/
    protected TProcessNode validateTwoNextNode(ProcessVo processVo, String currentNodeString, String id, String routeType, String auditType,
											   T entity, YifuUser loginUser) {
        List<TProcessNode> processNodeList = processVo.getProcessNodeList();
        // 判断当前节点类型,如果是选择节点则进行条件表达式判断判断
        TProcessNode currentNode = null;
        for (TProcessNode processNode : processNodeList) {
            if (StringUtils.equals(currentNodeString, processNode.getLogoTitle())) {
                currentNode = processNode;
            }
        }
        if(Common.isEmpty(currentNode)){
			throw new CheckedException("未获取到当前节点，请联系管理员处理");
		}
        // 1.7.5 未引入抄送逻辑：获取下个抄送节点信息（暂未引入）
        if (StringUtils.equals(ProcessConstants.PROCESS_NODE_TYPE_CHOICE, currentNode.getType())) {
            return getNextNode(processVo, currentNodeString, id, auditType, entity, loginUser);
        } else if (StringUtils.equals(ProcessConstants.PROCESS_NODE_SERVICE, currentNode.getType())) {
            if (StringUtils.equals(routeType, CommonConstants.ONE_STRING)) {
                // 是审批节点
                return currentNode;
            } else if (StringUtils.equals(routeType, CommonConstants.THREE_STRING)) {
                // 1.7.5 未引入抄送逻辑
                // 继续找下一个节点
				return getNextNode(processVo, currentNode.getLogoTitle(), id, auditType, entity, loginUser);
            } else {
                throw new CheckedException("未知的节点");
            }
        } else if (StringUtils.equals(ProcessConstants.PROCESS_NODE_BEGIN, currentNode.getType())) {
            return getNextNode(processVo, currentNode.getLogoTitle(), id, auditType, entity, loginUser);
        } else if (StringUtils.equals(ProcessConstants.PROCESS_NODE_END, currentNode.getType())) {
            return getNextNode(processVo, currentNode.getLogoTitle(), id, auditType, entity, loginUser);
        } else if (StringUtils.equals(ProcessConstants.PROCESS_NODE_EMBED, currentNode.getType())) {
            return getNextNode(processVo, currentNode.getLogoTitle(), id, auditType, entity, loginUser);
        } else {
            throw new CheckedException("未知的节点");
        }

    }

    /**
     * 保存经过流程节点记录
	 * @param processVo
     * @param nextNodeString
     * @param entityId
     **/
    protected void saveNodeProcess(ProcessVo processVo, String nextNodeString, String entityId, String type) {
        TProcessNode processNode = null;
        for (TProcessNode node : processVo.getProcessNodeList()) {
            if (StringUtils.equals(nextNodeString, node.getLogoTitle())) {
                processNode = node;
            }
        }
        if (processNode == null) {
            throw new CheckedException("节点id为空");
        }
        TAuditNodeProocess auditNodeProocess = new TAuditNodeProocess();
        saveProcessNodeProcess(processVo, entityId, type, processNode, auditNodeProocess);
    }

    /**
     * 保存经过流程节点记录
	 * @param processVo
     * @param entityId
     * @param processNode
     * @param auditNodeProocess
     **/
    private void saveProcessNodeProcess(ProcessVo processVo, String entityId, String type, TProcessNode processNode, TAuditNodeProocess auditNodeProocess) {
		auditNodeProocess.setDomainId(entityId);
		auditNodeProocess.setNodeId(processNode.getId());
		auditNodeProocess.setProcessId(processNode.getProcessId());
		auditNodeProocess.setCreateTime(LocalDateTime.now());

        // todo 新版流程审批默认主节点
		auditNodeProocess.setMainType(CommonConstants.ZERO_STRING);

		auditNodeProocess.setLogoTitle(processNode.getLogoTitle());
		auditNodeProocess.setNodeType(processNode.getType());
		auditNodeProocess.setNodeName(processNode.getName());
		auditNodeProocess.setProcessName(processVo.getName());
		auditNodeProocess.setDeleteFlag(CommonConstants.ZERO_STRING);
		auditNodeProocess.setType(type);
        auditNodeProocessService.save(auditNodeProocess);
    }
}
